% Step test for Model 30 - Boat
% Created by John Hedengren

clear all
close all

global u

% You can change these parameters *********************
% *****************************************************
% 1st time period - time (t1) and force (u1)
t1 = 16;
u1 = 1;

% 2nd time period - time (t2) and force (u2)
t2 = 11;
u2 = 0;

% 3rd time period - time (t3) and force (u3)
t3 = 1.1;
u3 = -1;

% 4th time period - time (t4) and force (u4)
t4 = 5;
u4 = 0;

% 5th time period - time (t5) and force (u5)
t5 = 50;
u5 = 0;

% End *************************************************
% *****************************************************



% Stack times
t2 = t1 + t2;
t3 = t2 + t3;
t4 = t3 + t4;
t5 = t4 + t5;


% Enforce constraints
u_upper_limit = 1;
u_lower_limit = -1;

% upper limits
if (u1 > u_upper_limit)
   u1 = u_upper_limit;
end
if (u2 > u_upper_limit)
   u2 = u_upper_limit;
end
if (u3 > u_upper_limit)
   u3 = u_upper_limit;
end
if (u4 > u_upper_limit)
   u4 = u_upper_limit;
end
if (u5 > u_upper_limit)
   u5 = u_upper_limit;
end

% lower limits
if (u1 < u_lower_limit)
   u1 = u_lower_limit;
end
if (u2 < u_lower_limit)
   u2 = u_lower_limit;
end
if (u3 < u_lower_limit)
   u3 = u_lower_limit;
end
if (u4 < u_lower_limit)
   u4 = u_lower_limit;
end
if (u5 < u_lower_limit)
   u5 = u_lower_limit;
end


% Initial Conditions for the States
pos_start = 100;
vel_start = 3;

% x_ss is a vector holding the initial states
x_ss = [pos_start; vel_start];


% Simulation with ode numerical integration

if (t1 <= 0)
   x_f1 = x_ss;
   t1_save = [];
   x1_save = [];
else
  	% Accelerate
	% Open Loop Step Change
	u = u1;
	[t1_save,x1_save] = ode15s('boat',[0 t1],x_ss);
	% Get final state for the next move
	x_f1 = x1_save(size(t1_save,1),:);
end

if (t2 <= t1)
   x_f2 = x_f1;
   t2_save = [];
   x2_save = [];   
else
	% Open Loop Step Change
	u = u2;
	[t2_save,x2_save] = ode15s('boat',[t1 t2],x_f1);
	% Get final state for the next move
	x_f2 = x2_save(size(t2_save,1),:);
end   

if (t3 <= t2)
   x_f3 = x_f2;
   t3_save = [];
   x3_save = [];   
else
	% Open Loop Step Change
	u = u3;
	[t3_save,x3_save] = ode15s('boat',[t2 t3],x_f2);
	% Get final state for the next move
	x_f3 = x3_save(size(t3_save,1),:);   
end

if (t4 <= t3)
   x_f4 = x_f3;
   t4_save = [];
   x4_save = [];   
else
	% Open Loop Step Change
	u = u4;
	[t4_save,x4_save] = ode15s('boat',[t3 t4],x_f3);
	% Get final state for the next move
	x_f4 = x4_save(size(t4_save,1),:);   
end

if (t5 <= t4)
   x_f5 = x_f4;
   t5_save = [];
   x5_save = [];   
else
	% Open Loop Step Change
	u = u5;
	[t5_save,x5_save] = ode15s('boat',[t4 t5],x_f4);
	% Get final state for the next move
	x_f5 = x5_save(size(t5_save,1),:);   
end





% Prepare the results for plotting
t = [t1_save;t2_save;t3_save;t4_save;t5_save];
y = [x1_save;x2_save;x3_save;x4_save;x5_save];
pos = y(:,1);
vel = y(:,2);

min_pos = min(pos)-10;
if (min_pos > -10)
   min_pos = -10;
end
max_pos = max(pos)+10;

% set up vertical lines
max_time = max(t);
min_vel = min(vel)-2;
max_vel = max(vel)+2;
vec1 = [min_vel; x_f1(1,2)];
vec2 = [min_vel; x_f2(1,2)];
vec3 = [min_vel; x_f3(1,2)];
vec4 = [min_vel; x_f4(1,2)];


t1_vec = [t1; t1];
t2_vec = [t2; t2];
t3_vec = [t3; t3];
t4_vec = [t4; t4];

figure(1);

subplot(2,1,1);
plot(t,pos);
hold on;
plot(t,zeros(size(t)),'r-');
ylabel('Position (m)');
legend('Boat position','Dock')
axis([0 max_time min_pos max_pos]);


subplot(2,1,2);
plot(t,vel);
hold on;
plot(t1_vec,vec1,'r-');
plot(t2_vec,vec2,'c-');
plot(t3_vec,vec3,'g-');
plot(t4_vec,vec4,'k-');
xlabel('time (sec)')
ylabel('Velocity (m/sec)');
axis([0 max_time min_vel max_vel]);
legend('Boat velocity','t2 start','t3 start','t4 start','t5 start')